/**
 * @file
 * Initiate inline documentation.
 */

(function ($) {
  'use strict';
  Drupal.behaviors.inline_documentation = {
    attach: function (context, settings) {
      var btn = $('.inline-documentation-button.toggle');
      var add_btn = $('.inline-documentation-button.add');
      var wrapper = $('.inline-documentation-wrapper');
      var entity = $('.inline-documentation-inline-documentation');
      var form = $('.inline-documentation-form-wrapper');
      var close_btn = $('.inline-documentation-form-wrapper .close-btn');
      var selector_help = $('#inline-documentation-selector-help');
      var overlay = $('#inline-documentation-selector-overlay');

      var max_wrapper_height = $(window).height() - btn.outerHeight() - (parseFloat(btn.css('bottom')) * 2);
      if (wrapper.hasClass('add-btn')) {
        max_wrapper_height -= add_btn.outerHeight() + parseFloat(btn.css('bottom'));
      }

      var padding = 0;
      var margin = 0;
      var i;
      var enable_selector = false;
      var cur;
      var no = [document.body, document.documentElement, document];

      if (entity.length > 0) {
        padding = entity.first().innerHeight() - entity.first().height();
        margin = entity.first().outerHeight(true) - entity.first().outerHeight();
      }

      wrapper.css('max-height', max_wrapper_height + 'px');

      repositionEntities();

      btn.mouseover(function (e) {
        if (!$(this).hasClass('open') && !add_btn.hasClass('form-open')) {
          openEntities();
          openButtons();
        }
      });

      btn.mouseleave(function (e) {
        if (!$(this).hasClass('open') || $(this).hasClass('clicked')) {
          return false;
        }

        if (e.offsetY < 0 && wrapper.length > 0 && wrapper.is(':visible')) {
          return false;
        }

        if (wrapper.length === 0 && e.offsetY < 0 && add_btn.length > 0) {
          return false;
        }

        closeEntities();
        closeButtons();
      });

      add_btn.mouseleave(function (e) {
        if (btn.hasClass('clicked')) {
          return false;
        }

        if (e.offsetY < 0 && wrapper.length > 0 && wrapper.is(':visible')) {
          return false;
        }

        if (e.offsetY >= $(this).outerHeight()) {
          return false;
        }

        closeEntities();
        closeButtons();
      });

      wrapper.mouseleave(function (e) {
        if (btn.hasClass('open') && !btn.hasClass('clicked')) {
          if (e.offsetY >= wrapper.height() && e.offsetX >= (wrapper.width() - btn.width() - 20)) {
            return;
          }

          closeEntities();
          closeButtons();
        }
      });

      btn.click(function (e) {
        if (add_btn.hasClass('form-open')) {
          closeForm();
        }

        if (!$(this).hasClass('clicked')) {
          $(this).addClass('clicked');

          if (!entity.is(':visible')) {
            openEntities();
          }

          if (!add_btn.is(':visible')) {
            add_btn.show().animate({
              bottom: '70px'
            }, 200);
          }

          if (!$(this).hasClass('open')) {
            $(this).addClass('open');
          }

          toggleContent(entity, 'show');
        }
        else {
          $(this).removeClass('clicked');
          toggleContent(entity, 'hide');
        }
        return false;
      });

      add_btn.click(function (e) {
        if (!$(this).hasClass('form-open')) {
          closeEntities();
          closeButtons();
          openForm();
        }
        return false;
      });

      close_btn.click(function (e) {
        closeForm();
        return false;
      });

      entity.find('.title').click(function (e) {
        toggleContent($(this).parent(), 'toggle');
        return false;
      });

      $('.inline-documentation-select-element').click(function (e) {
        form.fadeOut(200, function () {
          enable_selector = true;
        });
        selector_help.fadeIn();
        return false;
      });

      function openButtons() {
        btn.addClass('open');
        btn.find('span').css({'transform': 'rotate(360deg)'});

        add_btn.show().animate({
          bottom: '70px'
        }, 200);
      }

      function closeButtons() {
        btn.find('span').css({'transform': 'rotate(0deg)'});
        btn.removeClass('open').removeClass('clicked');

        add_btn.animate({
          bottom: btn.css('bottom')
        }, {
          duration: 200,
          queue: false
        }).fadeOut('fast');
      }

      function openForm() {
        var form_height = $(window).height() - btn.outerHeight() - (parseFloat(btn.css('bottom')) * 3);

        add_btn.addClass('form-open');

        form.show().animate({
          width: '500px',
          height: form_height + 'px'
        }, 200);
      }

      function closeForm() {
        add_btn.removeClass('form-open');
        btn.removeClass('clicked');

        form.animate({
          width: '1px',
          height: '1px'
        }, {
          duration: 200,
          queue: false
        }).fadeOut('fast');
      }

      function repositionEntities() {
        for (i = 0; i < entity.length; i++) {
          if (typeof $(entity[i]).attr('attach-to') !== 'undefined') {
            var element = $(entity[i]).attr('attach-to');
            var index = 0;
            if (element.substr(-1) === ']') {
              index = element.substr(element.lastIndexOf('['), element.length);
              index = index.substr(1, index.length - 2);
              element = element.substr(0, element.lastIndexOf('['));
              element = $(element)[index];
            }

            if ($(element).length > 0) {
              $(entity[i]).removeClass('element-not-exist');
              $(entity[i]).css('top', $(element).offset().top - $(entity[i]).height() - padding - margin);
              $(entity[i]).css('left', $(element).offset().left);
            }
            else {
              $(entity[i]).addClass('element-not-exist');
            }
          }
        }
      }

      function openEntities() {
        repositionEntities();


        entity.not('.element-not-exist').slideDown('fast');
        entity.not('.element-not-exist').animate({opacity: 1}, {duration: 500, queue: false});
      }

      function closeEntities() {
        toggleContent(entity, 'hide');
        entity.slideUp('fast');
        entity.animate({opacity: 0}, {duration: 100, queue: false}, 100);
      }

      function toggleContent(element, toggle) {
        for (i = 0; i < element.length; i++) {
          var content = $(element[i]).find('.content');

          if (content.is(':visible') && (toggle == 'toggle' || toggle == 'hide')) {
            content.slideUp('fast');
            element.removeClass('content-expanded');
          }
          else if (toggle == 'toggle' || toggle == 'show') {
            content.slideDown('fast');
            element.addClass('content-expanded');
          }
        }
      }

      $(document).mousemove(function(e) {
        if (enable_selector === false) {
          return;
        }

        if (e.target === cur) {
          return;
        }

        if (~no.indexOf(e.target)) {
          cur = null;
          overlay.hide();
          return;
        }

        var target = $(e.target),
          offset = target.offset(),
          width = target.outerWidth(),
          height = target.outerHeight();

        cur = e.target;

        overlay.css({
          top: offset.top,
          left: offset.left,
          width: width,
          height: height
        }).show();
      });

      $('a').click(function (e) {
        if (enable_selector === false) {
          return;
        }

        var id = getElementIdentifier($(this));
        $('#edit-element').val(id);
        enable_selector = false;
        overlay.hide();
        selector_help.fadeOut();
        form.fadeIn();

        e.stopImmediatePropagation();
        return false;
      });

      $(document).click(function (e) {
        if (enable_selector === false) {
          return;
        }

        var id = getElementIdentifier(e.target);
        $('#edit-element').val(id);
        enable_selector = false;
        overlay.hide();
        selector_help.fadeOut();
        form.fadeIn();

        e.stopImmediatePropagation();
        return false;
      });

      function getElementIdentifier(element) {
        var parents = $(element).parents();
        var i;
        var j;
        var id = '';
        var index = $(element).index();
        var class_blacklist = ['tableheader-processed', 'sticky-enabled'];
        var class_list;

        for (i = 0; i < parents.length; i++) {
          if (typeof $(parents[i]).attr('id') !== 'undefined' && $(parents[i]).attr('id') !== '') {
            id = '#' + $(parents[i]).attr('id') + ' ' + id;
            break;
          }
          if (typeof $(parents[i]).attr('class') !== 'undefined' && $(parents[i]).attr('class') !== '') {
            class_list = $(parents[i]).attr('class');

            // Filter out blacklisted classes.
            if (class_blacklist.length > 0) {
              for (j = 0; j < class_blacklist.length; j++) {
                class_list = class_list.replace(class_blacklist[j], '');
              }
            }

            // Replace multiple spaces with one space.
            class_list = class_list.replace(/\s\s+/g, ' ');
            // Remove surrounding spaces.
            class_list = $.trim(class_list);
            // Replace spaces with a dot and prepend the list with a dot.
            if (class_list.length > 0) {
              class_list = '.' + class_list.replace(/ /g, '.');
            }

            id = $(parents[i])[0].nodeName.toLowerCase() + class_list + ' ' + id;
          }
        }

        if (typeof $(element).attr('id') !== 'undefined' && $(element).attr('id') !== '') {
          id = '#' + $(element).attr('id');
        }
        else if (typeof $(element).attr('class') !== 'undefined' && $(element).attr('class') !== '') {
          id += $(parents[i])[0].nodeName.toLowerCase() + '.' + $(element).attr('class').replace(/ /g, '.') + '[' + index + ']';
        }
        else {
          id += $(element)[0].nodeName.toLowerCase() + '[' + index + ']';
        }

        if (id.length > 255) {
          id = id.substr(id.length - 255, id.length);
          id = id.substr(id.indexOf(' '), id.length);
        }

        return id;
      }

    }
  };
}(jQuery));